#pragma once

/**********************************************************************************************/
/*
   CIRCULARARRAY is a generic template class which implements a circular array that can store
   any arbitrary data type (ie double, DWORD) or struct that can be automatically copied, when
   the end of the array is reached, the oldest data is overwritten. 
*/

template <class DATA>
class CIRCULARARRAY
{
public:
   CIRCULARARRAY(int MaxSize);                                 // Creates a circular array which can store Size elements
   ~CIRCULARARRAY();
   void operator+=(DATA Data);                                 // Adds data to the array
   DATA operator[](int Pos);                                   // Retrieves a value from the array
   int GetLength()              { return m_Len;           };   // Retrieves the number of elements stored in the array
   int GetSize()                { return m_Size;          };   // Retrieves the max number of elements stored in the array
   bool IsFull()                { return m_Len >= m_Size; };   // Has the array filled out? 
   void Clear()                 { m_Len=0; m_Pos=0;       };   // Empties contents of the array
private:
   DATA* m_Array;
   int m_Len;
   int m_Pos;
   int m_Size;
};
/**********************************************************************************************/

/********************************************************************************************/
template <class DATA>
   CIRCULARARRAY<DATA>::CIRCULARARRAY(int Size)
{
   m_Array = new DATA[Size];
   memset(m_Array, 0, sizeof(DATA) * Size);
   m_Len = 0;
   m_Pos = 0;
   m_Size = Size;
};

template <class DATA>
   CIRCULARARRAY<DATA>::~CIRCULARARRAY()
{
   delete[] m_Array;
};

template <class DATA>
   void CIRCULARARRAY<DATA>::operator+=(DATA Data)
{
   m_Array[(m_Pos + m_Len) % m_Size] = Data;
   if (m_Len < m_Size)
      m_Len++;
   else
   {
      m_Pos++;
      if (m_Pos >= m_Size)
         m_Pos = 0;
   }
};


template <class DATA>
   DATA CIRCULARARRAY<DATA>::operator[](int Pos)
{
   ASSERT(m_Size);
   return m_Array[(m_Pos + Pos) % m_Size];
};
/********************************************************************************************/
